/*
 * prozeda.c
 *
 * Created: 05.02.2017 21:18:31
 *  Author: Chris
 */ 

#include <avr/pgmspace.h>
#include "prozeda.h"
#include "common.h"
#include "uart.h"

prozeda_data_t prozeda_data;

prozeda_msgcheck_t prozeda_checkMessage(uint8_t length, uint8_t expLength)
{
#ifdef PROZEDA_MSGCHK_DEBUG
	prozeda_data.msglen_exp = expLength;
	prozeda_data.msglen_act = length;
#endif

#ifdef PROZEDA_IGNORE_KITTY
	if(length == 1)
	{
		return prozeda_msgcheck_kitty;
	}
#endif

	//messages with length less 2 bytes are invalid
	if(expLength < 2 || expLength != length)
	{
		prozeda_statevt(rxerr_len);
		return prozeda_msgcheck_wrongLength;
	}
	
	//calculate the checksum
	uint8_t sum = 0;
	expLength--;
	uint8_t checksum = prozeda_data._buff[expLength];
	while(expLength--)
	{
		sum += prozeda_data._buff[expLength];
	}
	
#ifdef PROZEDA_MSGCHK_DEBUG
	prozeda_data.checksum_act = sum;
	prozeda_data.checksum_exp = checksum;
#endif
	
	if(sum != checksum)
	{
		prozeda_statevt(rxerr_checksum);
		return prozeda_msgcheck_wrongChecksum;
	}
	return prozeda_msgcheck_ok;
}

prozeda_taskstatus_t prozeda_task(void)
{
	if(prozeda_data._process == false)
	{
		return prozeda_taskstatus_noData;
	}
	//IOPIN_H(DBG2);
	prozeda_data._process = false;
	
	uint8_t len = prozeda_data._buffpos;
	prozeda_data._buffpos = 0;
	
	// disable and enable the SPI interface to flush it.
	// with this, data corruption due to missed/double clocks
	// "kitty packets" are avoided
	prozeda_spi_disable();
	prozeda_spi_enable();
	
	prozeda_taskstatus_t status = prozeda_taskstatus_noData;
	prozeda_msgcheck_t msgchkres = prozeda_msgcheck_unchecked;
	
	if(len == PROZEDA_MSGLEN_MSGTYPE)
	{
		// everything ok, now parse the data
		// check if it's a message type switcher
		if(arrcmp((const char*)prozeda_data._buff, "\xAA\x55\x55\xAA", 4))
		{
			prozeda_data._msgmode = (prozeda_data._buff[4] << 8) | prozeda_data._buff[5];
			status = prozeda_taskstatus_rxMsgtype;
		}
	}
	else if(prozeda_data._msgmode == prozeda_msgmode_logdata)
	{
		msgchkres = prozeda_checkMessage(len, PROZEDA_MSGLEN_LOGDATA);
		if(msgchkres != prozeda_msgcheck_ok)
			return (prozeda_taskstatus_t)msgchkres; 
		
		prozeda_statevt(msg_logdata);
		
		// if the pointer is defined, copy the data
		if(prozeda_data.logdata_ptr != NULL)
		{
			memcpy(prozeda_data.logdata_ptr, (const void*)prozeda_data._buff, PROZEDA_MSGLEN_LOGDATA);
			status = prozeda_taskstatus_rxLogdata;
		}
		// return another status if ptr is NULL?
	}
#ifdef PROZEDA_SUPPORT_DISPLAY
	else if(prozeda_data._msgmode == prozeda_msgmode_display)
	{
		//TODO: why is the checksum not checked here?
		msgchkres = prozeda_checkMessage(len, PROZEDA_MSGLEN_DISPLAY);
		if(msgchkres != prozeda_msgcheck_ok)
			return (prozeda_taskstatus_t)msgchkres;
		
		prozeda_statevt(msg_display);

		if(prozeda_data.displaydata_ptr != NULL)
		{
			memcpy(prozeda_data.displaydata_ptr, (const void*)prozeda_data._buff, PROZEDA_MSGLEN_DISPLAY);
			status = prozeda_taskstatus_rxDisplaydata;
		}
	}
#endif
#ifdef PROZEDA_SUPPORT_HEADER
	else if(prozeda_data._msgmode == prozeda_msgmode_header)
	{
		msgchkres = prozeda_checkMessage(len, PROZEDA_MSGLEN_HEADER);
		if(msgchkres != prozeda_msgcheck_ok)
			return (prozeda_taskstatus_t)msgchkres;
			
		prozeda_statevt(msg_header);

		if(prozeda_data.headerdata_ptr != NULL)
		{
			memcpy(prozeda_data.headerdata_ptr, (const void*)prozeda_data._buff, PROZEDA_MSGLEN_HEADER);
			status = prozeda_taskstatus_rxHeaderdata;
		}
	}
#endif

/*#ifdef PROZEDA_SUPPORT_SYSDATA
	else if(prozedata._msgmode == prozeda_msgmode_sysdata)
	{
		if(prozeda_checkMessage(len, PROZEDA_MSGLEN_SYSDATA))
		{
			if(prozedata.displaydata_ptr != NULL)
			{
				memcpy(prozedata.displaydata_ptr, (const void*)prozedata._buff, PROZEDA_MSGLEN_SYSDATA);
				status = prozeda_taskstatus_rxSysdata;
			}
		}
	}
#endif*/

	else
	{
		status = prozeda_taskstatus_rxNotImplemented;
	}
	//IOPIN_L(DBG2);
	return status;
}