/*
 * prozeda.h
 *
 * Created: 05.02.2017 21:19:33
 *  Author: Chris
 */ 

#include <avr/io.h>
#include <stdbool.h>
#include "prozeda_cfg.h"

#ifndef PROZEDA_H_
#define PROZEDA_H_


// was too lazy to do a better abstraction than this
#define prozeda_spi_disable() SPCR &= ~(1<<SPE)
#define prozeda_spi_enable() SPCR |= (1<<SPE)


//remove debug if not used
#ifndef prozeda_debug
	#define prozeda_debug(format, args...) /* there is no debug enabled */
#endif

#define PROZEDA_MSGLEN_MSGTYPE   6
#define PROZEDA_MSGLEN_REQUEST  33
#define PROZEDA_MSGLEN_DISPLAY  64
#define PROZEDA_MSGLEN_LOGDATA  68
#define PROZEDA_MSGLEN_HEADER   70
#define PROZEDA_MSGLEN_SYSDATA 260


#ifdef PROZEDA_SUPPORT_SYSDATA
#define PROZEDA_BUFF_LEN PROZEDA_MSGLEN_SYSDATA
#else
#define PROZEDA_BUFF_LEN PROZEDA_MSGLEN_HEADER
#endif

typedef enum
{
	prozeda_msgcheck_ok = 0,
	
	prozeda_msgcheck_kitty = 0xFB,
	prozeda_msgcheck_wrongLength = 0xFC,
	prozeda_msgcheck_wrongChecksum = 0xFD,
	
	prozeda_msgcheck_unchecked = 0xFF
}	prozeda_msgcheck_t;

typedef enum
{
	prozeda_msgmode_unknown = 0,
	prozeda_msgmode_display = 0x0100,
	prozeda_msgmode_request = 0x0200,
	prozeda_msgmode_logdata = 0x0300,
	prozeda_msgmode_header  = 0x0301,
}	prozeda_msgmode_t;

typedef enum
{
	prozeda_taskstatus_noData      = 0,
	prozeda_taskstatus_rxMsgtype = 1,
	prozeda_taskstatus_rxLogdata = 2,
	prozeda_taskstatus_rxDisplaydata = 3,
	prozeda_taskstatus_rxHeaderdata = 4,
	
	prozeda_taskstatus_rxNotImplemented = 0xF0,
	prozeda_taskstatus_kitty = 0xFB,
	prozeda_taskstatus_rxError = 0xFF,
	prozeda_taskstatus_rxErrorLen = 0xFC,
	prozeda_taskstatus_rxErrorChecksum = 0xFD,
	
}	prozeda_taskstatus_t;

#define PROZEDA_COLUMNNAME_LEN 14

typedef enum
{
	prozeda_columntype_dummy       = 0x00,
	prozeda_columntype_temperature = 0x01,
	prozeda_columntype_date        = 0x08,
	prozeda_columntype_time        = 0x09,
	prozeda_columntype_output      = 0x0A,
	prozeda_columntype_error1      = 0x0E,
	prozeda_columntype_error2      = 0x0F,
	prozeda_columntype_seconds     = 0x10,
	prozeda_columntype_throughput  = 0x13,
	prozeda_columntype_tapping     = 0x1B,
}	prozeda_columntype_t;

typedef struct
{
	#ifdef PROZEDA_USE_COLUMN_NAMES
	char name[PROZEDA_COLUMNNAME_LEN + 1]; //don't forget the terminator
	#endif
	prozeda_columntype_t type;
	uint8_t index;
}	prozeda_column_t;

// external variables

typedef struct
{
	// pointers
	uint8_t* logdata_ptr;
#ifdef PROZEDA_SUPPORT_DISPLAY
	uint8_t* displaydata_ptr;
#endif
#ifdef PROZEDA_SUPPORT_HEADER
	uint8_t* headerdata_ptr;
#endif
	
	// internal data
#ifdef PROZEDA_SUPPORT_SYSINFO
	volatile uint16_t _buffpos;
#else
	volatile uint8_t _buffpos;
#endif
	volatile uint8_t _buff[PROZEDA_BUFF_LEN];
	volatile uint8_t _ticks;
	volatile bool _process;
	prozeda_msgmode_t _msgmode;
	
	// debug data
#ifdef PROZEDA_MSGCHK_DEBUG
	uint8_t checksum_exp;
	uint8_t checksum_act;
	uint8_t msglen_exp;
	uint8_t msglen_act;
#endif

	// statistics (not yet implemented)
#ifdef PROZEDA_STATS
	uint32_t stats_msg_logdata;
	uint32_t stats_msg_display;
	uint32_t stats_msg_header;
	uint32_t stats_rxerr_len;
	uint32_t stats_rxerr_checksum;
#endif

}	prozeda_data_t;

extern prozeda_data_t prozeda_data;

#ifdef PROZEDA_STATS
#define prozeda_statevt(item) do { if(prozeda_data.stats_##item < UINT32_MAX) prozeda_data.stats_##item++; } while(0);
#else
#define prozeda_statevt(item)
#endif

prozeda_taskstatus_t prozeda_task(void);

inline void prozeda_tick(void)
{
	//IOPIN_H(DBG1);
	if(prozeda_data._ticks < 255)
	{
		prozeda_data._ticks++;
		if((prozeda_data._ticks >= PROZEDA_MSG_MAXTICKS) && (prozeda_data._buffpos > 0))
		{
			prozeda_data._process = true;
		}
	}
	//IOPIN_L(DBG1);
}

inline void prozeda_rxData(uint8_t data)
{
	//IOPIN_H(DBG0);
	uint8_t p = prozeda_data._buffpos;
	prozeda_data._ticks = 0;
	
	if(p < PROZEDA_BUFF_LEN)
	{
		prozeda_data._buff[p] = data;
		p++;
	}

	prozeda_data._buffpos = p;
	//IOPIN_L(DBG0);
}

#endif /* PROZEDA_H_ */