/*
 * bootinfo.c
 *
 * Created: 24.02.2019 21:50:30
 *  Author: Chris
 */ 

#include "sysinfo.h"
#include "uart.h"


#define RESET_MAGIC_SAFEWORD 0xDEADC0DE

uint32_t reset_magic __attribute__((section(".noinit")));
uint16_t reset_cnt __attribute__((section(".noinit")));
#ifdef PIROZEDA_UPTIME
uint32_t system_uptime __attribute__((section(".noinit")));
#endif

uint8_t system_mcusrdata = 0;


void system_resetinfo(bool afterdark)
{
	uart_write("Info: Last reset due to");
	if(afterdark)
	{
		system_mcusrdata = MCUSR;
		MCUSR = 0;
	}
	
	if(system_mcusrdata & (1<<WDRF))  uart_write(" WDT");
	if(system_mcusrdata & (1<<BORF))  uart_write(" BOR");
	if(system_mcusrdata & (1<<EXTRF)) uart_write(" EXT");
	if(system_mcusrdata & (1<<PORF))  uart_write(" POR");
	
	if(reset_magic != RESET_MAGIC_SAFEWORD)
	{
		reset_cnt = 0;
		reset_magic = RESET_MAGIC_SAFEWORD;
		uart_writeln(", no previous reboot");
	}
	else
	{
		uart_write(", reboot");
		if(reset_cnt < UINT16_MAX)
		{
			reset_cnt++;
			uart_write(" #%u", reset_cnt);
		}
		else
		{
			uart_write("ed very often");
		}
#ifdef PIROZEDA_UPTIME
		uart_writeln(", uptime was %u s", system_uptime);
#endif
	}
}

#ifdef PIROZEDA_UPTIME
uint32_t system_uptime_get(void)
{
	return system_uptime;	
}

void system_uptime_reset(void)
{
	system_uptime = 0;
}

void system_uptime_tick(void)
{
	system_uptime++;
}
#endif